@@ -25,9 +25,6 @@
 #include "../log_stream.h"
 #include "../mcon/manager.h"
 
-/* The Batch API is only available for 5.3.0+ */
-#if PHP_VERSION_ID >= 50300
-
 ZEND_EXTERN_MODULE_GLOBALS(mongo)
 
 extern zend_class_entry *mongo_ce_Collection;
@@ -126,6 +123,7 @@ PHP_METHOD(MongoWriteBatch, add)
 	php_mongo_write_update_args update_args = { NULL, NULL, -1, -1 };
 	php_mongo_write_delete_args delete_args = { NULL, -1 };
 	int status;
+	mongoclient *link;
 
 	intern = (mongo_write_batch_object*)zend_object_store_get_object(getThis() TSRMLS_CC);
 	MONGO_CHECK_INITIALIZED(intern->zcollection_object, MongoWriteBatch);
@@ -135,9 +133,11 @@ PHP_METHOD(MongoWriteBatch, add)
 	}
 
 	collection = (mongo_collection *)zend_object_store_get_object(intern->zcollection_object TSRMLS_CC);
-	connection = get_server(collection, MONGO_CON_FLAG_WRITE TSRMLS_CC);
+
+	link       = (mongoclient *)zend_object_store_get_object(collection->link TSRMLS_CC);
+	connection = php_mongo_collection_get_server(link, MONGO_CON_FLAG_WRITE TSRMLS_CC);
 	if (!connection) {
-		/* Exception thrown by get_server() */
+		/* Exception thrown by php_mongo_collection_get_server() */
 		return;
 	}
 
@@ -149,7 +149,7 @@ PHP_METHOD(MongoWriteBatch, add)
 	write_item.type = intern->batch_type;
 	switch (intern->batch_type) {
 		case MONGODB_API_COMMAND_INSERT:
-			write_item.write.insert = ht_item;
+			write_item.write.insert_doc = ht_item;
 			break;
 
 		case MONGODB_API_COMMAND_UPDATE: {
@@ -180,7 +180,7 @@ PHP_METHOD(MongoWriteBatch, add)
 				update_args.upsert = Z_BVAL_PP(upsert);
 			}
 
-			write_item.write.update = &update_args;
+			write_item.write.update_args = &update_args;
 			break;
 		 }
 
@@ -202,7 +202,7 @@ PHP_METHOD(MongoWriteBatch, add)
 			delete_args.query = *q;
 			delete_args.limit = Z_LVAL_PP(limit);
 
-			write_item.write.delete = &delete_args;
+			write_item.write.delete_args = &delete_args;
 			break;
 		 }
 
@@ -270,9 +270,7 @@ void php_mongo_writebatch_execute(mongo_write_batch_object *intern, mongo_connec
 		array_init(batch_retval);
 		status = php_mongo_api_batch_execute(batch, &intern->write_options, connection, &link->servers->options, batch_retval TSRMLS_CC);
 
-#if MONGO_PHP_STREAMS
 		mongo_log_stream_write_batch(connection, &intern->write_options, batch->request_id, batch_retval TSRMLS_CC);
-#endif
 
 		if (status) {
 			zval_ptr_dtor(&batch_retval);
@@ -409,9 +407,9 @@ PHP_METHOD(MongoWriteBatch, execute)
 	collection = (mongo_collection*)zend_object_store_get_object(intern->zcollection_object TSRMLS_CC);
 
 	link       = (mongoclient *)zend_object_store_get_object(collection->link TSRMLS_CC);
-	connection = get_server(collection, MONGO_CON_FLAG_WRITE TSRMLS_CC);
+	connection = php_mongo_collection_get_server(link, MONGO_CON_FLAG_WRITE TSRMLS_CC);
 	if (!connection) {
-		/* Exception thrown by get_server() */
+		/* Exception thrown by php_mongo_collection_get_server() */
 		return;
 	}
 
@@ -489,24 +487,24 @@ PHP_METHOD(MongoWriteBatch, getBatchInfo)
 }
 /* }}} */
 
-MONGO_ARGINFO_STATIC ZEND_BEGIN_ARG_INFO_EX(arginfo___construct, 0, ZEND_RETURN_VALUE, 2)
+ZEND_BEGIN_ARG_INFO_EX(arginfo___construct, 0, ZEND_RETURN_VALUE, 2)
 	ZEND_ARG_OBJ_INFO(0, collection, MongoCollection, 0)
 	ZEND_ARG_INFO(0, batch_type)
 	ZEND_ARG_ARRAY_INFO(0, write_options, 0)
 ZEND_END_ARG_INFO()
 
-MONGO_ARGINFO_STATIC ZEND_BEGIN_ARG_INFO_EX(arginfo_add, 0, ZEND_RETURN_VALUE, 1)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_add, 0, ZEND_RETURN_VALUE, 1)
 	ZEND_ARG_INFO(0, item)
 ZEND_END_ARG_INFO()
 
-MONGO_ARGINFO_STATIC ZEND_BEGIN_ARG_INFO_EX(arginfo_execute, 0, ZEND_RETURN_VALUE, 0)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_execute, 0, ZEND_RETURN_VALUE, 0)
 	ZEND_ARG_ARRAY_INFO(0, write_options, 0)
 ZEND_END_ARG_INFO()
 
-MONGO_ARGINFO_STATIC ZEND_BEGIN_ARG_INFO_EX(arginfo_getitemcount, 0, ZEND_RETURN_VALUE, 0)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_getitemcount, 0, ZEND_RETURN_VALUE, 0)
 ZEND_END_ARG_INFO()
 
-MONGO_ARGINFO_STATIC ZEND_BEGIN_ARG_INFO_EX(arginfo_getbatchinfo, 0, ZEND_RETURN_VALUE, 0)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_getbatchinfo, 0, ZEND_RETURN_VALUE, 0)
 ZEND_END_ARG_INFO()
 
 static zend_function_entry MongoWriteBatch_methods[] = {
@@ -531,8 +529,6 @@ void mongo_init_MongoWriteBatch(TSRMLS_D)
 	zend_declare_class_constant_long(mongo_ce_WriteBatch, "COMMAND_DELETE", strlen("COMMAND_DELETE"), MONGODB_API_COMMAND_DELETE TSRMLS_CC);
 }
 
-#endif /* PHP_VERSION_ID >= 50300 */
-
 /*
  * Local variables:
  * tab-width: 4
