@@ -37,8 +37,6 @@ ZEND_EXTERN_MODULE_GLOBALS(mongo)
 
 zend_class_entry *mongo_ce_CommandCursor = NULL;
 
-#define MONGO_DEFAULT_COMMAND_BATCH_SIZE 101
-
 void mongo_command_cursor_init(mongo_command_cursor *cmd_cursor, char *ns, zval *zlink, zval *zcommand TSRMLS_DC)
 {
 	mongoclient *link;
@@ -94,9 +92,13 @@ PHP_METHOD(MongoCommandCursor, __construct)
 }
 /* }}} */
 
-int php_mongo_enforce_batch_size_on_command(zval *command, int size TSRMLS_DC)
+/* Validates that the command's cursor field is an array or object if it exists.
+ * If it is an empty array, it will also be converted to an object to ensure
+ * proper BSON serialization (the server requires an object type). This function
+ * should be used after merging options into the command document. */
+int php_mongo_validate_cursor_on_command(zval *command TSRMLS_DC)
 {
-	zval **zcursor, **zbatchsize;
+	zval **zcursor;
 
 	if (Z_TYPE_P(command) != IS_ARRAY) {
 		/* Technically we can not hit this, as the command should be an
@@ -106,41 +108,53 @@ int php_mongo_enforce_batch_size_on_command(zval *command, int size TSRMLS_DC)
 	}
 
 	if (zend_hash_find(HASH_P(command), "cursor", 7, (void**) &zcursor) == FAILURE) {
-		zval *tmp_cursor;
-
-		MAKE_STD_ZVAL(tmp_cursor);
-		array_init(tmp_cursor);
-		zcursor = &tmp_cursor;
-
-		add_assoc_zval(command, "cursor", tmp_cursor);
+		return 1;
 	}
 
-	if (Z_TYPE_PP(zcursor) != IS_ARRAY) {
-		php_mongo_cursor_throw(mongo_ce_CursorException, NULL, 32 TSRMLS_CC, "The cursor command's 'cursor' element is not an array");
+	if (Z_TYPE_PP(zcursor) != IS_ARRAY && Z_TYPE_PP(zcursor) != IS_OBJECT) {
+		php_mongo_cursor_throw(mongo_ce_CursorException, NULL, 32 TSRMLS_CC, "The cursor command's 'cursor' element is not an array or object");
 		return 0;
 	}
 
-	if (zend_hash_find(HASH_PP(zcursor), "batchSize", 10, (void**) &zbatchsize) == FAILURE) {
-		add_assoc_long(*zcursor, "batchSize", size);
+	if (Z_TYPE_PP(zcursor) == IS_ARRAY && zend_hash_num_elements(Z_ARRVAL_PP(zcursor)) == 0) {
+		convert_to_object(*zcursor);
+	}
+
+	return 1;
+}
+
+/* Validates that the command's cursor field is an array or object. If it does
+ * not exist, it will be set with an empty object. This function should be used
+ * after merging options into the command document. */
+int php_mongo_enforce_cursor_on_command(zval *command TSRMLS_DC)
+{
+	if (!php_mongo_validate_cursor_on_command(command TSRMLS_CC)) {
+		return 0;
+	}
+
+	if (!zend_hash_exists(HASH_P(command), "cursor", 7)) {
+		zval *zcursor;
+
+		MAKE_STD_ZVAL(zcursor);
+		object_init(zcursor);
+
+		add_assoc_zval(command, "cursor", zcursor);
 	}
 
 	return 1;
 }
 
 /* {{{ MongoCommandCursor iteration helpers */
-static int php_mongocommandcursor_is_valid(mongo_command_cursor *cmd_cursor)
+int php_mongocommandcursor_is_valid(mongo_command_cursor *cmd_cursor)
 {
 	return cmd_cursor->current != NULL;
 }
 
-static int php_mongocommandcursor_load_current_element(mongo_command_cursor *cmd_cursor TSRMLS_DC)
+int php_mongocommandcursor_load_current_element(mongo_command_cursor *cmd_cursor TSRMLS_DC)
 {
 	/* Free the previous current item */
-	if (cmd_cursor->current) {
-		zval_ptr_dtor(&cmd_cursor->current);
-		cmd_cursor->current = NULL;
-	}
-	
+	php_mongo_cursor_clear_current_element(cmd_cursor);
+
 	/* Do processing of the first batch */
 	if (cmd_cursor->first_batch) {
 		zval **current;
@@ -158,8 +172,9 @@ static int php_mongocommandcursor_load_current_element(mongo_command_cursor *cmd
 	}
 	MAKE_STD_ZVAL(cmd_cursor->current);
 	array_init(cmd_cursor->current);
-	cmd_cursor->buf.pos = bson_to_zval(
+	cmd_cursor->buf.pos = (char*) bson_to_zval_iter(
 		(char*)cmd_cursor->buf.pos,
+		cmd_cursor->buf.end - cmd_cursor->buf.pos,
 		Z_ARRVAL_P(cmd_cursor->current),
 		NULL
 		TSRMLS_CC
@@ -172,16 +187,18 @@ static int php_mongocommandcursor_load_current_element(mongo_command_cursor *cmd
 	}
 
 	if (EG(exception)) {
-		zval_ptr_dtor(&cmd_cursor->current);
-		cmd_cursor->current = NULL;
+		php_mongo_cursor_clear_current_element(cmd_cursor);
 		return FAILURE;
 	}
 
 	return SUCCESS;
 }
 
-static int php_mongocommandcursor_advance(mongo_command_cursor *cmd_cursor TSRMLS_DC)
+int php_mongocommandcursor_advance(mongo_command_cursor *cmd_cursor TSRMLS_DC)
 {
+	/* Free the previous current item */
+	php_mongo_cursor_clear_current_element(cmd_cursor);
+
 	if (cmd_cursor->first_batch) {
 		cmd_cursor->first_batch_at++;
 
@@ -206,7 +223,6 @@ static int php_mongocommandcursor_advance(mongo_command_cursor *cmd_cursor TSRML
 				return FAILURE;
 			}
 			if (!php_mongo_get_more(cmd_cursor TSRMLS_CC)) {
-				cmd_cursor->cursor_id = 0;
 				return FAILURE;
 			}
 		}
@@ -215,7 +231,7 @@ static int php_mongocommandcursor_advance(mongo_command_cursor *cmd_cursor TSRML
 }
 /* }}} */
 
-static void fetch_batch_if_first_is_empty(mongo_cursor *cmd_cursor TSRMLS_DC)
+void php_mongocommandcursor_fetch_batch_if_first_is_empty(mongo_cursor *cmd_cursor TSRMLS_DC)
 {
 	if (cmd_cursor->first_batch_num == 0 && cmd_cursor->cursor_id != 0) {
 		zval_ptr_dtor(&cmd_cursor->first_batch);
@@ -252,7 +268,7 @@ PHP_METHOD(MongoCommandCursor, rewind)
 
 		/* If the first batch is empty (as it is with parallelCollectionScan), then
 		 * we already read the first batch here on rewind */
-		fetch_batch_if_first_is_empty(cmd_cursor TSRMLS_CC);
+		php_mongocommandcursor_fetch_batch_if_first_is_empty(cmd_cursor TSRMLS_CC);
 
 		php_mongocommandcursor_load_current_element(cmd_cursor TSRMLS_CC);
 
@@ -305,7 +321,7 @@ PHP_METHOD(MongoCommandCursor, rewind)
 
 	/* If the first batch is empty (as it is with parallelCollectionScan), then
 	 * we already read the first batch here on rewind */
-	fetch_batch_if_first_is_empty(cmd_cursor TSRMLS_CC);
+	php_mongocommandcursor_fetch_batch_if_first_is_empty(cmd_cursor TSRMLS_CC);
 
 	php_mongocommandcursor_load_current_element(cmd_cursor TSRMLS_CC);
 
@@ -438,7 +454,7 @@ void php_mongo_command_cursor_init_from_document(zval *zlink, mongo_command_curs
 	cmd_cursor->pre_created = 1;
 }
 
-static zval *php_mongo_commandcursor_instantiate(zval *object TSRMLS_DC)
+zval *php_mongo_commandcursor_instantiate(zval *object TSRMLS_DC)
 {
 	Z_TYPE_P(object) = IS_OBJECT;
 	object_init_ex(object, mongo_ce_CommandCursor);
@@ -485,25 +501,33 @@ PHP_METHOD(MongoCommandCursor, createFromDocument)
 }
 /* }}} */
 
-MONGO_ARGINFO_STATIC ZEND_BEGIN_ARG_INFO_EX(arginfo___construct, 0, ZEND_RETURN_VALUE, 2)
+ZEND_BEGIN_ARG_INFO_EX(arginfo___construct, 0, ZEND_RETURN_VALUE, 2)
 	ZEND_ARG_OBJ_INFO(0, connection, MongoClient, 0)
 	ZEND_ARG_INFO(0, database_and_collection_name)
 	ZEND_ARG_INFO(0, query)
 ZEND_END_ARG_INFO()
 
-MONGO_ARGINFO_STATIC ZEND_BEGIN_ARG_INFO_EX(arginfo_no_parameters, 0, ZEND_RETURN_VALUE, 0)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_no_parameters, 0, ZEND_RETURN_VALUE, 0)
 ZEND_END_ARG_INFO()
 
-MONGO_ARGINFO_STATIC ZEND_BEGIN_ARG_INFO_EX(arginfo_batchsize, 0, ZEND_RETURN_VALUE, 1)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_batchsize, 0, ZEND_RETURN_VALUE, 1)
 	ZEND_ARG_INFO(0, number)
 ZEND_END_ARG_INFO()
 
-MONGO_ARGINFO_STATIC ZEND_BEGIN_ARG_INFO_EX(arginfo_createfromdocument, 0, ZEND_RETURN_VALUE, 3)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_createfromdocument, 0, ZEND_RETURN_VALUE, 3)
 	ZEND_ARG_OBJ_INFO(0, connection, MongoClient, 0)
 	ZEND_ARG_INFO(0, connection_hash)
 	ZEND_ARG_INFO(0, cursor_document)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(arginfo_setReadPreference, 0, ZEND_RETURN_VALUE, 1)
+	ZEND_ARG_INFO(0, read_preference)
+	ZEND_ARG_ARRAY_INFO(0, tags, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_timeout, 0, ZEND_RETURN_VALUE, 1)
+	ZEND_ARG_INFO(0, timeoutMS)
+ZEND_END_ARG_INFO()
 
 static zend_function_entry MongoCommandCursor_methods[] = {
 	PHP_ME(MongoCommandCursor, __construct, arginfo___construct, ZEND_ACC_CTOR|ZEND_ACC_PUBLIC)
@@ -511,10 +535,15 @@ static zend_function_entry MongoCommandCursor_methods[] = {
 	/* options, code is reused from MongoCursor */
 	PHP_ME(MongoCursorInterface, batchSize, arginfo_batchsize, ZEND_ACC_PUBLIC)
 
-	/* query, code is shared through MongoCursorInterface */
+	/* timeout and diagnostics, code is shared through MongoCursorInterface */
+	PHP_ME(MongoCursorInterface, timeout, arginfo_timeout, ZEND_ACC_PUBLIC)
 	PHP_ME(MongoCursorInterface, info, arginfo_no_parameters, ZEND_ACC_PUBLIC)
 	PHP_ME(MongoCursorInterface, dead, arginfo_no_parameters, ZEND_ACC_PUBLIC)
 
+	/* read preferences, code is shared through MongoCursorInterface */
+	PHP_ME(MongoCursorInterface, getReadPreference, arginfo_no_parameters, ZEND_ACC_PUBLIC)
+	PHP_ME(MongoCursorInterface, setReadPreference, arginfo_setReadPreference, ZEND_ACC_PUBLIC)
+
 	/* iterator funcs */
 	PHP_ME(MongoCommandCursor, current, arginfo_no_parameters, ZEND_ACC_PUBLIC)
 	PHP_ME(MongoCommandCursor, key, arginfo_no_parameters, ZEND_ACC_PUBLIC)
