@@ -16,7 +16,7 @@
 #ifndef PHP_MONGO_H
 #define PHP_MONGO_H 1
 
-#define PHP_MONGO_VERSION "1.5.7"
+#define PHP_MONGO_VERSION "1.6.10"
 #define PHP_MONGO_EXTNAME "mongo"
 
 #ifdef HAVE_CONFIG_H
@@ -46,13 +46,6 @@
 # define Z_UNSET_ISREF_P(pz)      pz->is_ref = 0
 #endif
 
-#if PHP_VERSION_ID < 50300
-# define MAKE_COPY_ZVAL(ppzv, pzv) \
-    *(pzv) = **(ppzv);            \
-    zval_copy_ctor((pzv));        \
-    INIT_PZVAL((pzv));
-#endif
-
 #ifdef WIN32
 # ifndef int64_t
 typedef __int64 int64_t;
@@ -100,6 +93,8 @@ typedef __int64 int64_t;
 #define PHP_MONGO_DEFAULT_WTIMEOUT 10000
 #define PHP_MONGO_STATIC_CURSOR_TIMEOUT_NOT_SET_INITIALIZER -2
 
+#define PHP_MONGO_COLLECTION_DOES_NOT_EXIST 26
+
 /* if _id field should be added */
 #define PREP 1
 #define NO_PREP 0
@@ -111,49 +106,15 @@ typedef __int64 int64_t;
 #define DUP 1
 #define NO_DUP 0
 
-#define LAST_ERROR 0
-#define PREV_ERROR 1
-#define RESET_ERROR 2
-#define FORCE_ERROR 3
+#define PUSH_PARAM(arg) zend_vm_stack_push(arg TSRMLS_CC)
+#define POP_PARAM() (void)zend_vm_stack_pop(TSRMLS_C)
+#define PUSH_EO_PARAM()
+#define POP_EO_PARAM()
 
-#if PHP_VERSION_ID > 50300
-# define MONGO_ARGINFO_STATIC
-#else
-# define MONGO_ARGINFO_STATIC static
-#endif
-
-#if PHP_VERSION_ID >= 50300
-# define PUSH_PARAM(arg) zend_vm_stack_push(arg TSRMLS_CC)
-# define POP_PARAM() (void)zend_vm_stack_pop(TSRMLS_C)
-# define PUSH_EO_PARAM()
-# define POP_EO_PARAM()
-#else
-# define ZEND_VM_STACK_GROW_IF_NEEDED(x)
-# define PUSH_PARAM(arg) zend_ptr_stack_push(&EG(argument_stack), arg)
-# define POP_PARAM() (void)zend_ptr_stack_pop(&EG(argument_stack))
-# define PUSH_EO_PARAM() zend_ptr_stack_push(&EG(argument_stack), NULL)
-# define POP_EO_PARAM() (void)zend_ptr_stack_pop(&EG(argument_stack))
-#endif
-
-#if PHP_VERSION_ID >= 50300
-# define MONGO_E_DEPRECATED E_DEPRECATED
-#else
-# define MONGO_E_DEPRECATED E_STRICT
-#endif
 #if PHP_VERSION_ID < 50307
 # define PHP_FE_END         { NULL, NULL, NULL, 0, 0 }
 #endif
 
-#if PHP_VERSION_ID > 50300
-# define ERROR_HANDLER_DECLARATION(varname) zend_error_handling varname;
-# define ERROR_HANDLER_REPLACE(varname, ex) zend_replace_error_handling(EH_THROW, ex, &varname TSRMLS_CC)
-# define ERROR_HANDLER_RESTORE(varname)     zend_restore_error_handling(&varname TSRMLS_CC)
-#else
-# define ERROR_HANDLER_DECLARATION(varname)
-# define ERROR_HANDLER_REPLACE(varname, ex) php_set_error_handling(EH_THROW, ex TSRMLS_CC)
-# define ERROR_HANDLER_RESTORE(varname)     php_set_error_handling(EH_NORMAL, NULL TSRMLS_CC)
-#endif
-
 #define MUST_BE_ARRAY_OR_OBJECT(num, arg) do { \
 	if (arg && !(Z_TYPE_P(arg) == IS_ARRAY || Z_TYPE_P(arg) == IS_OBJECT)) { \
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "expects parameter %d to be an array or object, %s given", num, zend_get_type_by_const(Z_TYPE_P(arg))); \
@@ -161,26 +122,6 @@ typedef __int64 int64_t;
 	} \
 } while(0);
 
-#if PHP_VERSION_ID < 50300
-#define zpp_var_args(argv, argc) do { \
-	if (ZEND_NUM_ARGS() < 1) { \
-		WRONG_PARAM_COUNT; \
-	} \
-	argv = (zval ***)safe_emalloc(ZEND_NUM_ARGS(), sizeof(zval **), 0); \
-	if (zend_get_parameters_array_ex(ZEND_NUM_ARGS(), argv) == FAILURE) { \
-		efree(argv); \
-		WRONG_PARAM_COUNT; \
-	} \
-	argc = ZEND_NUM_ARGS(); \
-} while(0);
-#else
-#define zpp_var_args(argv, argc) do { \
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "+", &argv, &argc) == FAILURE) { \
-		return; \
-	} \
-} while(0);
-#endif
-
 #define MONGO_METHOD_BASE(classname, name) zim_##classname##_##name
 
 #define MONGO_METHOD_HELPER(classname, name, retval, thisptr, num, param) \
@@ -238,21 +179,6 @@ typedef __int64 int64_t;
 }
 #endif
 
-#define PHP_MONGO_TYPE_OBJ_NEW(mongo_obj)                    \
-	zend_object_value retval;                           \
-	mongo_obj *intern;                                  \
-	                                                    \
-	intern = (mongo_obj*)emalloc(sizeof(mongo_obj));               \
-	memset(intern, 0, sizeof(mongo_obj));                          \
-	                                                               \
-	zend_object_std_init(&intern->std, class_type TSRMLS_CC);      \
-	init_properties(intern);                                       \
-	                                                               \
-	retval.handle = zend_objects_store_put(intern,(zend_objects_store_dtor_t) zend_objects_destroy_object, php_##mongo_obj##_free, NULL TSRMLS_CC); \
-	retval.handlers = &mongo_type_object_handlers;                     \
-	                                                               \
-	return retval;
-
 #define PHP_MONGO_OBJ_NEW(mongo_obj)                    \
 	zend_object_value retval;                           \
 	mongo_obj *intern;                                  \
@@ -346,6 +272,12 @@ typedef struct {
 		RETURN_FALSE; \
 	}
 
+#define MONGO_CHECK_INITIALIZED_C(member, class_name) \
+	if (!(member)) { \
+		zend_throw_exception(mongo_ce_Exception, "The " #class_name " object has not been correctly initialized by its constructor", 0 TSRMLS_CC); \
+		return FAILURE; \
+	}
+
 #define MONGO_CHECK_INITIALIZED_STRING(member, class_name) \
 	if (!(member)) { \
 		zend_throw_exception(mongo_ce_Exception, "The " #class_name " object has not been correctly initialized by its constructor", 0 TSRMLS_CC); \
@@ -541,12 +473,10 @@ void mongo_init_MongoGridFS(TSRMLS_D);
 void mongo_init_MongoGridFSFile(TSRMLS_D);
 void mongo_init_MongoGridFSCursor(TSRMLS_D);
 
-#if PHP_VERSION_ID >= 50300
 void mongo_init_MongoWriteBatch(TSRMLS_D);
 void mongo_init_MongoInsertBatch(TSRMLS_D);
 void mongo_init_MongoUpdateBatch(TSRMLS_D);
 void mongo_init_MongoDeleteBatch(TSRMLS_D);
-#endif
 
 void mongo_init_MongoId(TSRMLS_D);
 void mongo_init_MongoCode(TSRMLS_D);
@@ -637,6 +567,9 @@ extern zend_module_entry mongo_module_entry;
  * 20: Cannot run command count(): (error message from MongoDB)
  * 21: Namespace field is invalid.
  * 22: invalid index specification
+ * 25: invalid RFC4122 UUID size
+ * 26: Invalid type of "filter" option for collection enumeration method
+ * 27: Invalid type of "filter" option "name" criteria for collection enumeration method on MongoDB <2.8
  *
  * MongoConnectionException:
  * 0: connection to <host> failed: <errmsg>
@@ -648,7 +581,7 @@ extern zend_module_entry mongo_module_entry;
  * 10: failed to get host from <substr> of <str>
  * 11: failed to get port from <substr> of <str>
  * 12: lost db connection
- * 2X: Parsing errors (ununsed)
+ * 2X: Parsing errors (unused)
  * 21: Empty option name or value
  * 22: Unknown connection string option
  * 23: Logical error (conflicting options)
@@ -687,10 +620,12 @@ extern zend_module_entry mongo_module_entry;
  * 18: Trying to get more, but cannot find server
  * 19: max number of retries exhausted, couldn't send query
  * 20: something exceptional has happened, and the cursor is now dead
+ * 21: Reading data for type %02x would exceed buffer for key "%s"
  * 21: invalid string length for key "%s"
  * 22: invalid binary length for key "%s"
  * 23: Can not natively represent the long %llu on this platform
  * 24: invalid code length for key "%s"
+ * 25: invalid RFC4122 UUID size
  * 28: recv_header() (abs()) recv_data() stream handlers error (timeout)
  * 29: Unknown query/get_more failure
  * 30: Cursor command response does not have the expected structure
@@ -700,6 +635,11 @@ extern zend_module_entry mongo_module_entry;
  * 35: Message size (%d) overflows valid message size (%d) php_mongo_api_get_reply()
  * 36: request/response mismatch: %d vs %d php_mongo_api_get_reply()
  * 37: Couldn't finish reading from network
+ * 38: Reading document length would exceed buffer (%u bytes)
+ * 39: Document length (%d bytes) should be at least 5 (i.e. empty document)
+ * 40: Document length (%d bytes) exceeds buffer (%u bytes)
+ * 41: string for key "%s" is not null-terminated
+ * 42: Document length (%u bytes) is not equal to buffer (%u bytes)
  *
  * MongoGridFSException:
  * 0: 
